"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tus_client_common_1 = require("./tus-client.common");
var UploadManager = (function () {
    function UploadManager() {
        var _this = this;
        this.worker = null;
        this.uploadQueue = [];
        this.currentUpload = null;
        var WorkerScript = require("nativescript-worker-loader!./android-worker.js");
        this.worker = new WorkerScript();
        this.worker.onmessage = function (msg) {
            if (!_this.currentUpload) {
                return;
            }
            _this.currentUpload.fromWorker(msg);
        };
        this.worker.onerror = function (error) {
            if (_this.currentUpload) {
                _this.currentUpload.workerError(error);
            }
            var WorkerScript = require("nativescript-worker-loader!./android-worker.js");
            _this.worker = new WorkerScript();
        };
    }
    UploadManager.singleton = function () {
        if (UploadManager.uploadManagerSingleton === null) {
            UploadManager.uploadManagerSingleton = new UploadManager();
        }
        return UploadManager.uploadManagerSingleton;
    };
    UploadManager.prototype.addUpload = function (upload) {
        this.uploadQueue.push(upload);
        this.processNextUpload();
    };
    UploadManager.prototype.sendWorker = function (msg) {
        this.worker.postMessage(msg);
    };
    UploadManager.prototype.uploadDone = function (upload) {
        if (this.currentUpload !== upload) {
            return;
        }
        this.currentUpload = null;
        this.processNextUpload();
    };
    UploadManager.prototype.processNextUpload = function () {
        if (!!this.currentUpload) {
            return;
        }
        var nextUpload = this.uploadQueue.shift();
        if (nextUpload) {
            this.currentUpload = nextUpload;
            this.currentUpload.workerStarted();
        }
    };
    UploadManager.uploadManagerSingleton = null;
    return UploadManager;
}());
var Upload = (function (_super) {
    __extends(Upload, _super);
    function Upload(file, options) {
        var _this = _super.call(this, file, options) || this;
        _this.abortResolve = null;
        _this.doAbort = false;
        return _this;
    }
    Upload.prototype.start = function () {
        UploadManager.singleton().addUpload(this);
    };
    Upload.prototype.abort = function () {
        var _this = this;
        this.doAbort = true;
        return new Promise(function (resolve) {
            _this.abortResolve = resolve;
        });
    };
    Upload.prototype.fromWorker = function (msg) {
        var action = msg.data.action;
        switch (action) {
            case 'chunkDone':
                this.options.onProgress(msg.data.progress.bytesSent, msg.data.progress.bytesTotal);
                if (this.doAbort) {
                    UploadManager.singleton().sendWorker({ action: 'abort' });
                }
                else {
                    UploadManager.singleton().sendWorker({ action: 'nextChunk' });
                }
                break;
            case 'done':
                if (msg.data.url) {
                    this.url = msg.data.url;
                    this.options.onSuccess();
                }
                if (this.doAbort && this.abortResolve) {
                    this.abortResolve('aborted');
                }
                UploadManager.singleton().uploadDone(this);
                break;
            case 'error':
                console.error(msg.data.error);
                this.options.onError(new Error(msg.data.error));
                break;
            default: break;
        }
    };
    Upload.prototype.workerStarted = function () {
        UploadManager.singleton().sendWorker({
            action: 'start',
            endpoint: this.options.endpoint,
            filepath: this.file.path,
            headers: this.options.headers,
            metadata: this.options.metadata
        });
    };
    Upload.prototype.workerError = function (error) {
        this.options.onError(new Error(error));
    };
    return Upload;
}(tus_client_common_1.UploadCommon));
exports.Upload = Upload;
//# sourceMappingURL=tus-client.android.js.map