"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("tns-core-modules/file-system");
var tus_client_common_1 = require("./tus-client.common");
var core_1 = require("@nativescript/core");
var Upload = (function (_super) {
    __extends(Upload, _super);
    function Upload(file, options) {
        var _this = _super.call(this, file, options) || this;
        var UPLOAD_PATH = "tus_upload";
        if (!core_1.File.exists(_this.file.path) || file.size === 0) {
            _this.options.onError(new Error("File " + _this.file + " does not exist or is empty"));
            return _this;
        }
        var URL = NSURL.URLWithString(fs.path.join(fs.knownFolders.documents().path, UPLOAD_PATH));
        _this.store = TUSFileUploadStore.alloc().initWithURL(URL);
        var fileUrl = NSURL.fileURLWithPath(_this.file.path);
        if (!fileUrl.checkResourceIsReachableAndReturnError()) {
            _this.options.onError(new Error("File " + _this.file + " not reachable"));
            return _this;
        }
        var uploadEndpoint = NSURL.URLWithString(_this.options.endpoint);
        var session = TUSSession.alloc().initWithEndpointDataStoreAllowsCellularAccess(uploadEndpoint, _this.store, true);
        _this.tusSession = session.createUploadFromFileRetryHeadersMetadata(fileUrl, 3, _this.toDictionary(options.headers), _this.toDictionary(options.metadata));
        if (!_this.tusSession) {
            _this.options.onError(new Error('could not create upload object'));
        }
        _this.tusSession.progressBlock = function (bytesSent, bytesTotal) {
            _this.options.onProgress(bytesSent, bytesTotal);
        };
        _this.tusSession.resultBlock = function (url) {
            _this.url = url.absoluteString;
            _this.options.onSuccess();
        };
        _this.tusSession.failureBlock = function (error) {
            _this.options.onError(new Error(error.localizedDescription));
        };
        return _this;
    }
    Upload.prototype.start = function () {
        this.tusSession.resume();
    };
    Upload.prototype.abort = function () {
        this.tusSession.stop();
        return Promise.resolve();
    };
    Upload.prototype.toDictionary = function (obj) {
        var node = NSMutableDictionary.new();
        for (var property in obj) {
            if (obj.hasOwnProperty(property)
                && obj[property] != null
                && typeof obj[property] === 'string') {
                node.setObjectForKey(String(obj[property]), property);
            }
        }
        return node;
    };
    return Upload;
}(tus_client_common_1.UploadCommon));
exports.Upload = Upload;
//# sourceMappingURL=tus-client.ios.js.map